// Copyright (C) 2010 - DIGITEO - Michael Baudin
// Copyright (C) 2000 - John Burkardt
// Copyright (C) 2000-2004 - Benoit Hamelin, Jean-Pierre Dussault
// Copyright (C) 1994 - Chaya Gurwitz, Livia Klein, Madhu Lamba
// Copyright (C) 1981 - More, Garbow, Hillstrom
//
// This file must be used under the terms of the GNU LGPL license.

function [fopt,xopt] = uncprb_getopt(nprob,n,m)
  // Returns the name.
  //
  // Calling Sequence
  //   [fopt,xopt] = uncprb_getopt(nprob,n,m)
  //
  // Parameters
  //   nprob: a floating point integers, the problem number
  //   n: the number of variables, i.e. the size of x
  //   m: the number of functions, i.e. the size of fvec
  //   fopt : a 1 x 1 matrix of doubles, the minimum of the function
  //   xopt : a matrix of doubles, n x 1, the optimum of the function
  //
  // Description
  // Returns the optimum, according to the paper "Algo 566".
  // For each problem this corresponds to the data in the (d) section.
  // For some problems, the optimum is known only for particular values of m and n.
  // If fopt or xopt is unknown, an error is returned.
  //
  // When the paper mentions several local optimums, we return 
  // the first one in the paper. When the paper only gives 
  // a limited number of significant digits, so do we.
  //
  // When the paper writes +/-inf, we set +/-%inf.
  //
  // When the paper gives the function value, but not the optimum,
  // we return xopt=[]. This allows to return something,
  // instead of nothing at all.
  //
  // There are some cases where the optimum is known for several
  // particular values of n or m. This is why n and m are input 
  // arguments. If it turns out that the optimum is not 
  // known for this particular value of n or m, we generate 
  // an error.
  //
  // We emphasize that, when xopt is unknown, there is no
  // way to check that fopt is correct, that is satisfies 
  // f(xopt)=fopt and g(xopt)=0.
  //
  // All default settings provided by getinitf do correspond to 
  // the parameters used here. This allows to make checkings with 
  // the default settings, when possible.
  //
  // For problem #20 - Watson - the values of the approximate solutions are taken from Brent,
  // "Algorithms for Minimization with Derivatives", Prentice-Hall, 1973.
  // The values are taken from John Burkardt - "test_opt" - March 2000.
  // I improved the precision based on a full-precision optimization with optim : the 
  // gradient is smaller.
  //
  // For problem #3 - Powell Badly Scaled - a manual optimization was done
  // and the results are reported to full precision. We claim
  // that the gradient is exactly zero at optimum.
  //
  // Examples
  //   // Get optimum for Rosenbrock's test case
  //   nprob = 1;
  //   [n,m,x0]=uncprb_getinitf(nprob);
  //   [fopt,xopt] = uncprb_getopt(nprob,n,m)
  //   // Check that the optimum is known is this case
  //   isknown = ( xopt <> [] )
  //
  //   // Check what is provided for problem #8
  //   nprob = 8;
  //   [n,m,x0]=uncprb_getinitf(nprob);
  //   [fopt,xopt] = uncprb_getopt(nprob,n,m)
  //   // See that fopt is known, but not xopt
  //   isknown = ( xopt <> [] )
  //
  // Authors
  //   Michael Baudin - 2010 - DIGITEO
  //   Scilab port: 2000-2004, Benoit Hamelin, Jean-Pierre Dussault
  //   Optimum for problem #20 - 2000 - John Burkardt
  //   Matlab port: 1994, Chaya Gurwitz, Livia Klein, Madhu Lamba
  //   Fortran 77: 1981, More, Garbow, Hillstrom
  
  [lhs,rhs]=argn();
  if ( rhs <> 3 ) then
    errmsg = msprintf(gettext("%s: Unexpected number of input arguments : %d provided while 3 are expected."), "uncprb_getname", rhs);
    error(errmsg)
  end
  //
  if ( nprob==1 ) then
    fopt = 0
    xopt = [1 1]'
  elseif ( nprob==2 ) then
    fopt = 0
    xopt = [5 4]'
  elseif ( nprob==3 ) then
    fopt = 0
    xopt = [1.098159329699795501D-05  9.106146739866707307D+00]'
  elseif ( nprob==4 ) then
    fopt = 0
    xopt = [1.e6 2.e-6]'
  elseif ( nprob==5 ) then
    fopt = 0
    xopt = [3 0.5]'
  elseif ( nprob==6 ) then
    apifun_checktrue ( m==10 , msprintf(gettext("%s: Expected m==10, but got the contrary. m=%d"), "uncprb_getopt" , m ))
    fopt = 124.362
    xopt = [0.2578 0.2578]'
  elseif ( nprob==7 ) then
    fopt = 0
    xopt = [1 0 0]'
  elseif ( nprob==8 ) then
    fopt = 8.21487e-3
    xopt = []
  elseif ( nprob==9 ) then
    fopt = 1.12793e-8
    xopt = []
  elseif ( nprob==10 ) then
    fopt = 87.9458
    xopt = []
  elseif ( nprob==11 ) then
    fopt = 0
    xopt = [50 25 1.5]'
  elseif ( nprob==12 ) then
    fopt = 0
    xopt = [1 10 1]'
  elseif ( nprob==13 ) then
    fopt = 0
    xopt = zeros(n,1)
  elseif ( nprob==14 ) then
    fopt = 0
    xopt = [1 1 1 1]'
  elseif ( nprob==15 ) then
    fopt = 3.07505e-4
    xopt = []
  elseif ( nprob==16 ) then
    apifun_checktrue ( m==20 , msprintf(gettext("%s: Expected m==20, but got the contrary. m=%d"), "uncprb_getopt" , m ))
    fopt = 85822.2
    xopt = []
  elseif ( nprob==17 ) then
    fopt = 5.46489e-5
    xopt = []
  elseif ( nprob==18 ) then
    apifun_checktrue ( m==13 , msprintf(gettext("%s: Expected m==13, but got the contrary. m=%d"), "uncprb_getopt" , m ))
    fopt = 6.65565e-3
    xopt = []
  elseif ( nprob==19 ) then
    fopt = 4.01377e-2
    xopt = []
  elseif ( nprob==20 ) then
    apifun_checktrue ( n==6 | n==9 | n==12, msprintf(gettext("%s: Expected n==9 | n==12, but got the contrary. n=%d"), "uncprb_getopt" , n ))
    if ( n==6 ) then
      fopt = 2.28767e-3
      xopt = [-0.015725D+00, 1.012435D+00, -0.232992D+00, 1.260430D+00, -1.513729D+00, 0.992996D+00]'
    elseif ( n==9 ) then
      fopt = 1.39976e-6
      xopt = [-1.530703669113499455D-05 9.997897039319447732D-01 1.476396369401976848D-02  1.463423282932473646D-01 1.000821103030963277D+00 -2.617731140577795657D+00 4.104403164548480198D+00 -3.143612278598149512D+00 1.052626408019991633D+00]'
    elseif ( n==12 ) then
      fopt = 4.72238e-10
      xopt = []
    end
  elseif ( nprob==21 ) then
    fopt = 0
    xopt = ones(n,1)
  elseif ( nprob==22 ) then
    fopt = 0
    xopt = zeros(n,1)
  elseif ( nprob==23 ) then
    apifun_checktrue ( n==4 | n==10, msprintf(gettext("%s: Expected n==4 | n==10, but got the contrary. n=%d"), "uncprb_getopt" , n ))
    if ( n==4 ) then
      fopt = 2.24997e-5
      xopt = []
    elseif ( n==10 ) then
      fopt = 7.08765e-5
      xopt = []
    end
  elseif ( nprob==24 ) then
    apifun_checktrue ( n==4 | n==10, msprintf(gettext("%s: Expected n==4 | n==10, but got the contrary. n=%d"), "uncprb_getopt" , n ))
    if ( n==4 ) then
      fopt = 9.37629e-6
      xopt = []
    elseif ( n==10 ) then
      fopt = 2.93660e-4
      xopt = []
    end
  elseif ( nprob==25 ) then
    fopt = 0
    xopt = ones(n,1)
  elseif ( nprob==26 ) then
    fopt = 0
    xopt = []
  elseif ( nprob==27 ) then
    fopt = 0
    xopt = ones(n,1)
  elseif ( nprob==28 ) then
    fopt = 0
    xopt = []
  elseif ( nprob==29 ) then
    fopt = 0
    xopt = []
  elseif ( nprob==30 ) then
    fopt = 0
    xopt = []
  elseif ( nprob==31 ) then
    fopt = 0
    xopt = []
  elseif ( nprob==32 ) then
    fopt = m-n
    xopt = -ones(n,1)
  elseif ( nprob==33 ) then
    fopt = ( m*(m-1) ) / ( 2*(2*m+1) )
    xopt = []
  elseif ( nprob==34 ) then
    fopt = (m^2+3*m-6)/(2*(2*m-3))
    xopt = []
  elseif ( nprob==35 ) then
    apifun_checktrue ( m==n , msprintf(gettext("%s: Expected m==n, but got the contrary. n=%d, m=%d"), "uncprb_getopt" , n , m ))
    apifun_checktrue ( n==8 | n==10, msprintf(gettext("%s: Expected n==8 | n==10, but got the contrary. n=%d"), "uncprb_getopt" , n ))
    if ( n==4 ) then
      fopt = 3.51687e-3
      xopt = []
    elseif ( n==10 ) then
      fopt = 6.50395e-3
      xopt = []
    end
  else
    error(msprintf("%s: The input argument nprob is %d but should be between 1 and 35.\n","uncprb_getname",nprob))
  end
endfunction
function apifun_checktrue ( var , msg )
  if ( ~var ) then
    error(msg);
  end
endfunction

